@@ -43,12 +43,6 @@
  * with #GstGLVideoAllocationParams.
  *
  * Data is uploaded or downloaded from the GPU as is necessary.
- *
- * The #GstCaps that is used for #GstGLMemory based buffers should contain
- * the %GST_CAPS_FEATURE_MEMORY_GL_MEMORY as a #GstCapsFeatures and should
- * contain a 'texture-target' field with one of the #GstGLTextureTarget values
- * as a string, i.e. some combination of 'texture-target=(string){2D,
- * rectangle, external-oes}'.
  */
 
 #define USING_OPENGL(context) (gst_gl_context_check_gl_version (context, GST_GL_API_OPENGL, 1, 0))
@@ -59,7 +53,7 @@
 
 #define GL_MEM_WIDTH(gl_mem) _get_plane_width (&gl_mem->info, gl_mem->plane)
 #define GL_MEM_HEIGHT(gl_mem) _get_plane_height (&gl_mem->info, gl_mem->plane)
-#define GL_MEM_STRIDE(gl_mem) _get_mem_stride (gl_mem)
+#define GL_MEM_STRIDE(gl_mem) GST_VIDEO_INFO_PLANE_STRIDE (&gl_mem->info, gl_mem->plane)
 
 static GstAllocator *_gl_memory_allocator;
 
@@ -71,10 +65,6 @@ GST_DEBUG_CATEGORY_STATIC (GST_CAT_GL_MEMORY);
 #define GL_UNPACK_ROW_LENGTH 0x0CF2
 #endif
 
-#ifndef GL_PACK_ROW_LENGTH
-#define GL_PACK_ROW_LENGTH 0x0D02
-#endif
-
 #ifndef GL_TEXTURE_RECTANGLE
 #define GL_TEXTURE_RECTANGLE 0x84F5
 #endif
@@ -92,10 +82,6 @@ GST_DEBUG_CATEGORY_STATIC (GST_CAT_GL_MEMORY);
 G_DEFINE_TYPE (GstGLMemoryAllocator, gst_gl_memory_allocator,
     GST_TYPE_GL_BASE_MEMORY_ALLOCATOR);
 
-#ifndef GST_REMOVE_DEPRECATED
-GST_DEFINE_MINI_OBJECT_TYPE (GstGLMemory, gst_gl_memory);
-#endif
-
 typedef struct
 {
   /* in */
@@ -111,62 +97,32 @@ typedef struct
 } GstGLMemoryCopyParams;
 
 static inline guint
-_get_plane_width (const GstVideoInfo * info, guint plane)
+_get_plane_width (GstVideoInfo * info, guint plane)
 {
-  if (GST_VIDEO_INFO_IS_YUV (info)) {
-    if (GST_VIDEO_INFO_FORMAT (info) == GST_VIDEO_FORMAT_v210) {
-      return (guint) (((guint64) (info->width + 5) / 6) * 4);
-    } else {
-      gint comp[GST_VIDEO_MAX_COMPONENTS];
-      gst_video_format_info_component (info->finfo, plane, comp);
-      return GST_VIDEO_INFO_COMP_WIDTH (info, comp[0]);
-    }
-  } else {
-    /* RGB, GRAY */
+  if (GST_VIDEO_INFO_IS_YUV (info))
+    /* For now component width and plane width are the same and the
+     * plane-component mapping matches
+     */
+    return GST_VIDEO_INFO_COMP_WIDTH (info, plane);
+  else                          /* RGB, GRAY */
     return GST_VIDEO_INFO_WIDTH (info);
-  }
 }
 
 static inline guint
-_get_plane_height (const GstVideoInfo * info, guint plane)
+_get_plane_height (GstVideoInfo * info, guint plane)
 {
-  if (GST_VIDEO_FORMAT_INFO_IS_TILED (info->finfo)) {
-    guint tile_height;
-    gsize stride;
-
-    tile_height = GST_VIDEO_FORMAT_INFO_TILE_HEIGHT (info->finfo, plane);
-    stride = GST_VIDEO_INFO_PLANE_STRIDE (info, plane);
-
-    return GST_VIDEO_TILE_Y_TILES (stride) * tile_height;
-  }
-
-  if (GST_VIDEO_INFO_IS_YUV (info)) {
-    gint comp[GST_VIDEO_MAX_COMPONENTS];
-    gst_video_format_info_component (info->finfo, plane, comp);
-    return GST_VIDEO_INFO_COMP_HEIGHT (info, comp[0]);
-  }
-
-  /* RGB, GRAY */
-  return GST_VIDEO_INFO_HEIGHT (info);
-}
-
-static guint
-_get_mem_stride (GstGLMemory * gl_mem)
-{
-  const GstVideoFormatInfo *finfo = gl_mem->info.finfo;
-  guint stride = GST_VIDEO_INFO_PLANE_STRIDE (&gl_mem->info, gl_mem->plane);
-
-  if (!GST_VIDEO_FORMAT_INFO_IS_TILED (finfo))
-    return stride;
-
-  return GST_VIDEO_TILE_X_TILES (stride) *
-      GST_VIDEO_FORMAT_INFO_TILE_STRIDE (finfo, gl_mem->plane);
+  if (GST_VIDEO_INFO_IS_YUV (info))
+    /* For now component width and plane width are the same and the
+     * plane-component mapping matches
+     */
+    return GST_VIDEO_INFO_COMP_HEIGHT (info, plane);
+  else                          /* RGB, GRAY */
+    return GST_VIDEO_INFO_HEIGHT (info);
 }
 
 static inline void
 _calculate_unpack_length (GstGLMemory * gl_mem, GstGLContext * context)
 {
-  GstVideoInfo *info = &gl_mem->info;
   guint n_gl_bytes;
   GstGLFormat tex_format;
   guint tex_type;
@@ -174,6 +130,7 @@ _calculate_unpack_length (GstGLMemory * gl_mem, GstGLContext * context)
   gl_mem->tex_scaling[0] = 1.0f;
   gl_mem->tex_scaling[1] = 1.0f;
   gl_mem->unpack_length = 1;
+  gl_mem->tex_width = GL_MEM_WIDTH (gl_mem);
 
   gst_gl_format_type_from_sized_gl_format (gl_mem->tex_format, &tex_format,
       &tex_type);
@@ -183,13 +140,6 @@ _calculate_unpack_length (GstGLMemory * gl_mem, GstGLContext * context)
     return;
   }
 
-  /* For tiles, we need GL not to clip on the display width, as the would make
-   * some tile not fully accessible by GLSL. */
-  if (GST_VIDEO_FORMAT_INFO_IS_TILED (info->finfo))
-    gl_mem->tex_width = GL_MEM_STRIDE (gl_mem) / n_gl_bytes;
-  else
-    gl_mem->tex_width = GL_MEM_WIDTH (gl_mem);
-
   if (USING_OPENGL (context) || USING_GLES3 (context)
       || USING_OPENGL3 (context)) {
     gl_mem->unpack_length = GL_MEM_STRIDE (gl_mem) / n_gl_bytes;
@@ -358,8 +308,8 @@ _gst_gl_memory_end_log (GstGLMemory * gl_mem)
 void
 gst_gl_memory_init (GstGLMemory * mem, GstAllocator * allocator,
     GstMemory * parent, GstGLContext * context, GstGLTextureTarget target,
-    GstGLFormat tex_format, const GstAllocationParams * params,
-    const GstVideoInfo * info, guint plane, const GstVideoAlignment * valign,
+    GstGLFormat tex_format, GstAllocationParams * params,
+    GstVideoInfo * info, guint plane, GstVideoAlignment * valign,
     gpointer user_data, GDestroyNotify notify)
 {
   const gchar *target_str;
@@ -410,10 +360,10 @@ gst_gl_memory_init (GstGLMemory * mem, GstAllocator * allocator,
 /**
  * gst_gl_memory_read_pixels:
  * @gl_mem: a #GstGLMemory
- * @write_pointer: the data pointer to pass to glReadPixels
+ * @read_pointer: the data pointer to pass to glReadPixels
  *
- * Reads the texture in #GstGLMemory into @write_pointer if no buffer is bound
- * to `GL_PIXEL_PACK_BUFFER`.  Otherwise @write_pointer is the byte offset into
+ * Reads the texture in #GstGLMemory into @read_pointer if no buffer is bound
+ * to `GL_PIXEL_PACK_BUFFER`.  Otherwise @read_pointer is the byte offset into
  * the currently bound `GL_PIXEL_PACK_BUFFER` buffer to store the result of
  * glReadPixels.  See the OpenGL specification for glReadPixels for more
  * details.
@@ -423,7 +373,7 @@ gst_gl_memory_init (GstGLMemory * mem, GstAllocator * allocator,
  * Since: 1.8
  */
 gboolean
-gst_gl_memory_read_pixels (GstGLMemory * gl_mem, gpointer write_pointer)
+gst_gl_memory_read_pixels (GstGLMemory * gl_mem, gpointer read_pointer)
 {
   GstGLContext *context = gl_mem->mem.context;
   const GstGLFuncs *gl = context->gl_vtable;
@@ -467,16 +417,9 @@ gst_gl_memory_read_pixels (GstGLMemory * gl_mem, gpointer write_pointer)
     }
   }
 
-  if (USING_OPENGL (context) || USING_GLES3 (context)
-      || USING_OPENGL3 (context)) {
-    gl->PixelStorei (GL_PACK_ROW_LENGTH, gl_mem->unpack_length);
-  } else if (USING_GLES2 (context)) {
-    gl->PixelStorei (GL_PACK_ALIGNMENT, gl_mem->unpack_length);
-  }
-
   _gst_gl_memory_start_log (gl_mem, "glReadPixels");
   gl->ReadPixels (0, 0, gl_mem->tex_width, GL_MEM_HEIGHT (gl_mem), format,
-      type, write_pointer);
+      type, read_pointer);
   _gst_gl_memory_end_log (gl_mem);
 
   gl->BindFramebuffer (GL_FRAMEBUFFER, 0);
@@ -575,8 +518,6 @@ _upload_cpu_write (GstGLMemory * gl_mem, GstMapInfo * info, gsize maxsize)
  * @gl_mem: a #GstGLMemory
  * @read_pointer: the data pointer to pass to glTexSubImage
  *
- * Reads the texture in @read_pointer into @gl_mem.
- *
  * See gst_gl_memory_read_pixels() for what @read_pointer signifies.
  *
  * Since: 1.8
@@ -1041,7 +982,7 @@ gst_gl_memory_allocator_init (GstGLMemoryAllocator * allocator)
  * @width: width of @tex_id
  * @height: height of @tex_id
  *
- * Copies @gl_mem into the texture specified by @tex_id.  The format of @tex_id
+ * Copies @gl_mem into the texture specfified by @tex_id.  The format of @tex_id
  * is specified by @tex_format, @width and @height.
  *
  * Returns: Whether the copy succeeded
@@ -1158,7 +1099,7 @@ gst_gl_memory_get_texture_id (GstGLMemory * gl_mem)
 void
 gst_gl_memory_init_once (void)
 {
-  static gsize _init = 0;
+  static volatile gsize _init = 0;
 
   if (g_once_init_enter (&_init)) {
     gst_gl_base_memory_init_once ();
@@ -1197,7 +1138,7 @@ G_DEFINE_BOXED_TYPE (GstGLVideoAllocationParams, gst_gl_video_allocation_params,
 
 static void
 _gst_gl_video_allocation_params_set_video_alignment (GstGLVideoAllocationParams
-    * params, const GstVideoAlignment * valign)
+    * params, GstVideoAlignment * valign)
 {
   g_return_if_fail (params != NULL);
 
@@ -1240,8 +1181,8 @@ gboolean
 gst_gl_video_allocation_params_init_full (GstGLVideoAllocationParams * params,
     gsize struct_size, guint alloc_flags, GstGLAllocationParamsCopyFunc copy,
     GstGLAllocationParamsFreeFunc free, GstGLContext * context,
-    const GstAllocationParams * alloc_params, const GstVideoInfo * v_info,
-    guint plane, const GstVideoAlignment * valign, GstGLTextureTarget target,
+    GstAllocationParams * alloc_params, GstVideoInfo * v_info,
+    guint plane, GstVideoAlignment * valign, GstGLTextureTarget target,
     GstGLFormat tex_format, gpointer wrapped_data, gpointer gl_handle,
     gpointer user_data, GDestroyNotify notify)
 {
@@ -1290,8 +1231,8 @@ gst_gl_video_allocation_params_init_full (GstGLVideoAllocationParams * params,
  */
 GstGLVideoAllocationParams *
 gst_gl_video_allocation_params_new (GstGLContext * context,
-    const GstAllocationParams * alloc_params, const GstVideoInfo * v_info,
-    guint plane, const GstVideoAlignment * valign, GstGLTextureTarget target,
+    GstAllocationParams * alloc_params, GstVideoInfo * v_info, guint plane,
+    GstVideoAlignment * valign, GstGLTextureTarget target,
     GstGLFormat tex_format)
 {
   GstGLVideoAllocationParams *params = g_new0 (GstGLVideoAllocationParams, 1);
@@ -1331,8 +1272,8 @@ gst_gl_video_allocation_params_new (GstGLContext * context,
  */
 GstGLVideoAllocationParams *
 gst_gl_video_allocation_params_new_wrapped_data (GstGLContext * context,
-    const GstAllocationParams * alloc_params, const GstVideoInfo * v_info,
-    guint plane, const GstVideoAlignment * valign, GstGLTextureTarget target,
+    GstAllocationParams * alloc_params, GstVideoInfo * v_info, guint plane,
+    GstVideoAlignment * valign, GstGLTextureTarget target,
     GstGLFormat tex_format, gpointer wrapped_data, gpointer user_data,
     GDestroyNotify notify)
 {
@@ -1378,8 +1319,8 @@ gst_gl_video_allocation_params_new_wrapped_data (GstGLContext * context,
  */
 GstGLVideoAllocationParams *
 gst_gl_video_allocation_params_new_wrapped_gl_handle (GstGLContext * context,
-    const GstAllocationParams * alloc_params, const GstVideoInfo * v_info,
-    guint plane, const GstVideoAlignment * valign, GstGLTextureTarget target,
+    GstAllocationParams * alloc_params, GstVideoInfo * v_info, guint plane,
+    GstVideoAlignment * valign, GstGLTextureTarget target,
     GstGLFormat tex_format, gpointer gl_handle, gpointer user_data,
     GDestroyNotify notify)
 {
@@ -1421,8 +1362,8 @@ gst_gl_video_allocation_params_new_wrapped_gl_handle (GstGLContext * context,
  */
 GstGLVideoAllocationParams *
 gst_gl_video_allocation_params_new_wrapped_texture (GstGLContext * context,
-    const GstAllocationParams * alloc_params, const GstVideoInfo * v_info,
-    guint plane, const GstVideoAlignment * valign, GstGLTextureTarget target,
+    GstAllocationParams * alloc_params, GstVideoInfo * v_info, guint plane,
+    GstVideoAlignment * valign, GstGLTextureTarget target,
     GstGLFormat tex_format, guint tex_id, gpointer user_data,
     GDestroyNotify notify)
 {
