@@ -236,6 +236,10 @@ forced_key_unit_event_compare (const ForcedKeyUnitEvent * a,
 static GstElementClass *parent_class = NULL;
 static gint private_offset = 0;
 
+/* cached quark to avoid contention on the global quark table lock */
+#define META_TAG_VIDEO meta_tag_video_quark
+static GQuark meta_tag_video_quark;
+
 static void gst_video_encoder_class_init (GstVideoEncoderClass * klass);
 static void gst_video_encoder_init (GstVideoEncoder * enc,
     GstVideoEncoderClass * klass);
@@ -418,6 +422,8 @@ gst_video_encoder_class_init (GstVideoEncoderClass * klass)
           "Minimum interval between force-keyunit requests in nanoseconds", 0,
           G_MAXUINT64, DEFAULT_MIN_FORCE_KEY_UNIT_INTERVAL,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  meta_tag_video_quark = g_quark_from_static_string (GST_META_TAG_VIDEO_STR);
 }
 
 static GList *
@@ -2137,8 +2143,7 @@ gst_video_encoder_transform_meta_default (GstVideoEncoder *
   tags = gst_meta_api_type_get_tags (info->api);
 
   if (!tags || (g_strv_length ((gchar **) tags) == 1
-          && gst_meta_api_type_has_tag (info->api,
-              g_quark_from_string (GST_META_TAG_VIDEO_STR))))
+          && gst_meta_api_type_has_tag (info->api, META_TAG_VIDEO)))
     return TRUE;
 
   return FALSE;
