@@ -265,7 +265,6 @@ struct _GstDecodebin3
 
   /* Properties */
   GstCaps *caps;
-  gboolean force_sw_decoders;
 };
 
 struct _GstDecodebin3Class
@@ -372,8 +371,7 @@ typedef struct _PendingPad
 enum
 {
   PROP_0,
-  PROP_CAPS,
-  PROP_FORCE_SW_DECODERS,
+  PROP_CAPS
 };
 
 /* signals */
@@ -425,8 +423,6 @@ G_DEFINE_TYPE (GstDecodebin3, gst_decodebin3, GST_TYPE_BIN);
 
 static GstStaticCaps default_raw_caps = GST_STATIC_CAPS (DEFAULT_RAW_CAPS);
 
-#define DEFAULT_FORCE_SW_DECODERS FALSE
-
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -555,20 +551,6 @@ gst_decodebin3_class_init (GstDecodebin3Class * klass)
           "The caps on which to stop decoding. (NULL = default)",
           GST_TYPE_CAPS, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  /**
-   * GstDecodeBin::force-sw-decoders:
-   *
-   * While auto-plugging, if set to %TRUE, those decoders within
-   * "Hardware" klass will be ignored. Otherwise they will be tried.
-   *
-   * Since: 1.18
-   */
-  g_object_class_install_property (gobject_klass, PROP_FORCE_SW_DECODERS,
-      g_param_spec_boolean ("force-sw-decoders", "Software Decoders Only",
-          "Use only sofware decoders to process streams",
-          DEFAULT_FORCE_SW_DECODERS,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
   /* FIXME : ADD SIGNALS ! */
   /**
    * GstDecodebin3::select-stream
@@ -579,6 +561,10 @@ gst_decodebin3_class_init (GstDecodebin3Class * klass)
    * This signal is emitted whenever @decodebin needs to decide whether
    * to expose a @stream of a given @collection.
    *
+   * Note that the prefered way to select streams is to listen to
+   * GST_MESSAGE_STREAM_COLLECTION on the bus and send a
+   * GST_EVENT_SELECT_STREAMS with the streams the user wants.
+   *
    * Returns: 1 if the stream should be selected, 0 if it shouldn't be selected.
    * A value of -1 (default) lets @decodebin decide what to do with the stream.
    * */
@@ -645,7 +631,6 @@ gst_decodebin3_init (GstDecodebin3 * dbin)
   g_mutex_init (&dbin->input_lock);
 
   dbin->caps = gst_static_caps_get (&default_raw_caps);
-  dbin->force_sw_decoders = DEFAULT_FORCE_SW_DECODERS;
 
   GST_OBJECT_FLAG_SET (dbin, GST_BIN_FLAG_STREAMS_AWARE);
 }
@@ -697,9 +682,6 @@ gst_decodebin3_set_property (GObject * object, guint prop_id,
       dbin->caps = g_value_dup_boxed (value);
       GST_OBJECT_UNLOCK (dbin);
       break;
-    case PROP_FORCE_SW_DECODERS:
-      dbin->force_sw_decoders = g_value_get_boolean (value);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -719,9 +701,6 @@ gst_decodebin3_get_property (GObject * object, guint prop_id, GValue * value,
       g_value_set_boxed (value, dbin->caps);
       GST_OBJECT_UNLOCK (dbin);
       break;
-    case PROP_FORCE_SW_DECODERS:
-      g_value_set_boolean (value, dbin->force_sw_decoders);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -922,6 +901,7 @@ gst_decodebin3_input_pad_unlink (GstPad * pad, GstObject * parent)
       if (dbin->collection)
         gst_object_unref (dbin->collection);
       dbin->collection = collection;
+      dbin->select_streams_seqnum = GST_SEQNUM_INVALID;
 
       msg =
           gst_message_new_stream_collection ((GstObject *) dbin,
@@ -1062,19 +1042,12 @@ gst_decode_bin_update_factories_list (GstDecodebin3 * dbin)
     dbin->decodable_factories = NULL;
     for (tmp = dbin->factories; tmp; tmp = tmp->next) {
       GstElementFactory *fact = (GstElementFactory *) tmp->data;
-
       if (gst_element_factory_list_is_type (fact,
-              GST_ELEMENT_FACTORY_TYPE_DECODER)) {
-        if (!(dbin->force_sw_decoders
-                && gst_element_factory_list_is_type (fact,
-                    GST_ELEMENT_FACTORY_TYPE_HARDWARE))) {
-          dbin->decoder_factories =
-              g_list_append (dbin->decoder_factories, fact);
-        }
-      } else {
+              GST_ELEMENT_FACTORY_TYPE_DECODER))
+        dbin->decoder_factories = g_list_append (dbin->decoder_factories, fact);
+      else
         dbin->decodable_factories =
             g_list_append (dbin->decodable_factories, fact);
-      }
     }
   }
 }
@@ -1161,14 +1134,15 @@ update_requested_selection (GstDecodebin3 * dbin)
   }
 
   /* 4. If the user didn't explicitly selected all streams, match one stream of each type */
-  if (!all_user_selected && !dbin->requested_selection) {
+  if (!all_user_selected && dbin->select_streams_seqnum == GST_SEQNUM_INVALID) {
     for (i = 0; i < nb; i++) {
       GstStream *stream = gst_stream_collection_get_stream (collection, i);
       GstStreamType curtype = gst_stream_get_stream_type (stream);
       if (!(used_types & curtype)) {
         const gchar *sid = gst_stream_get_stream_id (stream);
-        GST_DEBUG_OBJECT (dbin, "Selecting stream '%s' of type %s",
-            sid, gst_stream_type_get_name (curtype));
+        GST_DEBUG_OBJECT (dbin,
+            "Automatically selecting stream '%s' of type %s", sid,
+            gst_stream_type_get_name (curtype));
         tmp = g_list_append (tmp, (gchar *) sid);
         used_types |= curtype;
       }
@@ -1453,6 +1427,7 @@ handle_stream_collection (GstDecodebin3 * dbin,
     /* dbin->pending_collection = */
     /*     g_list_append (dbin->pending_collection, collection); */
   }
+  dbin->select_streams_seqnum = GST_SEQNUM_INVALID;
   SELECTION_UNLOCK (dbin);
 }
 
