@@ -19,6 +19,7 @@
 
 /**
  * SECTION:element-videorate
+ * @title: videorate
  *
  * This element takes an incoming stream of timestamped video frames.
  * It will produce a perfect stream that matches the source pad's framerate.
@@ -56,20 +57,22 @@
  * certain factor. It must not be confused with framerate. Think of rate as
  * speed and framerate as flow.
  *
- * <refsect2>
- * <title>Example pipelines</title>
+ * ## Example pipelines
  * |[
  * gst-launch-1.0 -v uridecodebin uri=file:///path/to/video.ogg ! videoconvert ! videoscale ! videorate ! video/x-raw,framerate=15/1 ! autovideosink
- * ]| Decode a video file and adjust the framerate to 15 fps before playing.
+ * ]|
+ *  Decode a video file and adjust the framerate to 15 fps before playing.
  * To create a test Ogg/Theora file refer to the documentation of theoraenc.
  * |[
  * gst-launch-1.0 -v v4l2src ! videorate ! video/x-raw,framerate=25/2 ! theoraenc ! oggmux ! filesink location=recording.ogg
- * ]| Capture video from a V4L device, and adjust the stream to 12.5 fps before
+ * ]|
+ *  Capture video from a V4L device, and adjust the stream to 12.5 fps before
  * encoding to Ogg/Theora.
  * |[
  * gst-launch-1.0 -v uridecodebin uri=file:///path/to/video.ogg ! videoconvert ! videoscale ! videorate ! video/x-raw,framerate=1/5 ! jpegenc ! multifilesink location=snapshot-%05d.jpg
- * ]| Decode a video file and save a snapshot every 5 seconds as consecutively numbered jpeg file.
- * </refsect2>
+ * ]|
+ *  Decode a video file and save a snapshot every 5 seconds as consecutively numbered jpeg file.
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -96,6 +99,7 @@ enum
 #define DEFAULT_AVERAGE_PERIOD  0
 #define DEFAULT_MAX_RATE        G_MAXINT
 #define DEFAULT_RATE            1.0
+#define DEFAULT_MAX_DUPLICATION_TIME      0
 
 enum
 {
@@ -110,7 +114,8 @@ enum
   PROP_DROP_ONLY,
   PROP_AVERAGE_PERIOD,
   PROP_MAX_RATE,
-  PROP_RATE
+  PROP_RATE,
+  PROP_MAX_DUPLICATION_TIME
 };
 
 static GstStaticPadTemplate gst_video_rate_src_template =
@@ -272,6 +277,22 @@ gst_video_rate_class_init (GstVideoRateClass * klass)
           DEFAULT_RATE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS |
           GST_PARAM_MUTABLE_READY));
 
+  /**
+   * GstVideoRate:max-duplication-time:
+   *
+   * Duplicate frames only if the gap between two consecutive frames does not
+   * exceed this duration.
+   *
+   * Since: 1.16
+   */
+  g_object_class_install_property (object_class, PROP_MAX_DUPLICATION_TIME,
+      g_param_spec_uint64 ("max-duplication-time",
+          "Maximum time to duplicate a frame",
+          "Do not duplicate frames if the gap exceeds this period "
+          "(in ns) (0 = disabled)",
+          0, G_MAXUINT64, DEFAULT_MAX_DUPLICATION_TIME,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   gst_element_class_set_static_metadata (element_class,
       "Video rate adjuster", "Filter/Effect/Video",
       "Drops/duplicates/adjusts timestamps on video frames to make a perfect stream",
@@ -454,7 +475,7 @@ gst_video_rate_transform_caps (GstBaseTransform * trans,
           gst_structure_set (s3, "framerate", GST_TYPE_FRACTION, 0, 1, NULL);
         }
       } else if (max_num != 0 || max_denom != 1) {
-        /* We can provide everything upto the maximum framerate at the src */
+        /* We can provide everything up to the maximum framerate at the src */
         gst_structure_set (s2, "framerate", GST_TYPE_FRACTION_RANGE,
             0, 1, max_num, max_denom, NULL);
       }
@@ -547,7 +568,8 @@ gst_video_rate_setcaps (GstBaseTransform * trans, GstCaps * in_caps,
    * out_frame_count */
   if (videorate->to_rate_numerator) {
     videorate->base_ts +=
-        gst_util_uint64_scale (videorate->out_frame_count,
+        gst_util_uint64_scale (videorate->out_frame_count +
+        (videorate->segment.rate < 0.0 ? 1 : 0),
         videorate->to_rate_denominator * GST_SECOND,
         videorate->to_rate_numerator);
   }
@@ -611,6 +633,8 @@ gst_video_rate_init (GstVideoRate * videorate)
   videorate->average_period_set = DEFAULT_AVERAGE_PERIOD;
   videorate->max_rate = DEFAULT_MAX_RATE;
   videorate->rate = DEFAULT_RATE;
+  videorate->pending_rate = DEFAULT_RATE;
+  videorate->max_duplication_time = DEFAULT_MAX_DUPLICATION_TIME;
 
   videorate->from_rate_numerator = 0;
   videorate->from_rate_denominator = 0;
@@ -620,25 +644,14 @@ gst_video_rate_init (GstVideoRate * videorate)
   gst_base_transform_set_gap_aware (GST_BASE_TRANSFORM (videorate), TRUE);
 }
 
-/* flush the oldest buffer */
+/* @outbuf: (transfer full) needs to be writable */
 static GstFlowReturn
-gst_video_rate_flush_prev (GstVideoRate * videorate, gboolean duplicate,
-    GstClockTime next_intime)
+gst_video_rate_push_buffer (GstVideoRate * videorate, GstBuffer * outbuf,
+    gboolean duplicate, GstClockTime next_intime)
 {
   GstFlowReturn res;
-  GstBuffer *outbuf;
   GstClockTime push_ts;
 
-  if (!videorate->prevbuf)
-    goto eos_before_buffers;
-
-  outbuf = gst_buffer_ref (videorate->prevbuf);
-  if (videorate->drop_only)
-    gst_buffer_replace (&videorate->prevbuf, NULL);
-
-  /* make sure we can write to the metadata */
-  outbuf = gst_buffer_make_writable (outbuf);
-
   GST_BUFFER_OFFSET (outbuf) = videorate->out;
   GST_BUFFER_OFFSET_END (outbuf) = videorate->out + 1;
 
@@ -661,14 +674,22 @@ gst_video_rate_flush_prev (GstVideoRate * videorate, gboolean duplicate,
   if (videorate->segment.rate < 0.0) {
     if (videorate->to_rate_numerator) {
       /* interpolate next expected timestamp in the segment */
-
-      videorate->next_ts =
+      GstClockTimeDiff next_ts =
           videorate->segment.base + videorate->segment.stop -
           videorate->base_ts -
+          gst_util_uint64_scale (videorate->out_frame_count + 1,
+          videorate->to_rate_denominator * GST_SECOND,
+          videorate->to_rate_numerator);
+
+      videorate->next_ts = next_ts < 0 ? GST_CLOCK_TIME_NONE : next_ts;
+
+      GST_BUFFER_DURATION (outbuf) =
           gst_util_uint64_scale (videorate->out_frame_count,
+          videorate->to_rate_denominator * GST_SECOND,
+          videorate->to_rate_numerator) -
+          gst_util_uint64_scale (videorate->out_frame_count - 1,
           videorate->to_rate_denominator * GST_SECOND,
           videorate->to_rate_numerator);
-      GST_BUFFER_DURATION (outbuf) = push_ts - videorate->next_ts;
     } else if (next_intime != GST_CLOCK_TIME_NONE) {
       videorate->next_ts = next_intime;
     } else {
@@ -684,13 +705,15 @@ gst_video_rate_flush_prev (GstVideoRate * videorate, gboolean duplicate,
           videorate->to_rate_denominator * GST_SECOND,
           videorate->to_rate_numerator);
       GST_BUFFER_DURATION (outbuf) = videorate->next_ts - push_ts;
-    } else if (GST_CLOCK_TIME_IS_VALID (GST_BUFFER_DURATION (outbuf))) {
-      videorate->next_ts =
-          GST_BUFFER_PTS (outbuf) + GST_BUFFER_DURATION (outbuf);
     } else {
       /* There must always be a valid duration on prevbuf if rate > 0,
        * it is ensured in the transform_ip function */
-      GST_FIXME_OBJECT (videorate, "No buffer duration known");
+      g_assert (GST_BUFFER_PTS_IS_VALID (outbuf));
+      g_assert (GST_BUFFER_DURATION_IS_VALID (outbuf));
+      g_assert (GST_BUFFER_DURATION (outbuf) != 0);
+
+      videorate->next_ts
+          = GST_BUFFER_PTS (outbuf) + GST_BUFFER_DURATION (outbuf);
     }
   }
 
@@ -707,6 +730,23 @@ gst_video_rate_flush_prev (GstVideoRate * videorate, gboolean duplicate,
   res = gst_pad_push (GST_BASE_TRANSFORM_SRC_PAD (videorate), outbuf);
 
   return res;
+}
+
+/* flush the oldest buffer */
+static GstFlowReturn
+gst_video_rate_flush_prev (GstVideoRate * videorate, gboolean duplicate,
+    GstClockTime next_intime)
+{
+  GstBuffer *outbuf;
+
+  if (!videorate->prevbuf)
+    goto eos_before_buffers;
+
+  outbuf = gst_buffer_ref (videorate->prevbuf);
+  /* make sure we can write to the metadata */
+  outbuf = gst_buffer_make_writable (outbuf);
+
+  return gst_video_rate_push_buffer (videorate, outbuf, duplicate, next_intime);
 
   /* WARNINGS */
 eos_before_buffers:
@@ -818,11 +858,13 @@ gst_video_rate_sink_event (GstBaseTransform * trans, GstEvent * event)
 
       break;
     }
+    case GST_EVENT_SEGMENT_DONE:
     case GST_EVENT_EOS:{
       gint count = 0;
       GstFlowReturn res = GST_FLOW_OK;
 
-      GST_DEBUG_OBJECT (videorate, "Got EOS");
+      GST_DEBUG_OBJECT (videorate, "Got %s",
+          gst_event_type_get_name (GST_EVENT_TYPE (event)));
 
       /* If the segment has a stop position, fill the segment */
       if (GST_CLOCK_TIME_IS_VALID (videorate->segment.stop)) {
@@ -840,9 +882,8 @@ gst_video_rate_sink_event (GstBaseTransform * trans, GstEvent * event)
                     && GST_CLOCK_TIME_IS_VALID (videorate->next_ts)
                     && videorate->next_ts - videorate->segment.base >=
                     videorate->segment.start)
-                || count < 1)) {
-          res =
-              gst_video_rate_flush_prev (videorate, count > 0,
+            )) {
+          res = gst_video_rate_flush_prev (videorate, count > 0,
               GST_CLOCK_TIME_NONE);
           count++;
         }
@@ -875,7 +916,8 @@ gst_video_rate_sink_event (GstBaseTransform * trans, GstEvent * event)
         videorate->dup += count - 1;
         if (!videorate->silent)
           gst_video_rate_notify_duplicate (videorate);
-      } else if (count == 0) {
+      } else if (count == 0
+          && !GST_CLOCK_TIME_IS_VALID (videorate->segment.stop)) {
         videorate->drop++;
         if (!videorate->silent)
           gst_video_rate_notify_drop (videorate);
@@ -941,6 +983,43 @@ gst_video_rate_src_event (GstBaseTransform * trans, GstEvent * event)
       res = gst_pad_push_event (sinkpad, event);
       break;
     }
+    case GST_EVENT_QOS:
+    {
+      GstQOSType type;
+      gdouble proportion;
+      GstClockTimeDiff diff;
+      GstClockTime timestamp;
+
+      gst_event_parse_qos (event, &type, &proportion, &diff, &timestamp);
+
+      if (GST_CLOCK_TIME_IS_VALID (timestamp) && videorate->rate != 1.0) {
+        GST_OBJECT_LOCK (trans);
+        GST_DEBUG_OBJECT (trans, "Rescaling QoS event taking our rate into"
+            "account. Timestamp:  %" GST_TIME_FORMAT " -> %" GST_TIME_FORMAT
+            " - diff %" G_GINT64_FORMAT "-> %" G_GINT64_FORMAT,
+            GST_TIME_ARGS (timestamp),
+            GST_TIME_ARGS (videorate->base_ts + ((timestamp -
+                        videorate->base_ts) * videorate->rate)), diff,
+            (GstClockTimeDiff) (diff * videorate->rate));
+
+        if (videorate->segment.rate < 0.0)
+          timestamp =
+              (videorate->segment.stop - videorate->base_ts) -
+              ((videorate->segment.stop - videorate->base_ts -
+                  timestamp) * videorate->rate);
+        else
+          timestamp =
+              videorate->base_ts + ((timestamp -
+                  videorate->base_ts) * videorate->rate);
+
+        diff *= videorate->rate;
+        GST_OBJECT_UNLOCK (trans);
+
+        gst_event_unref (event);
+        event = gst_event_new_qos (type, proportion, diff, timestamp);
+      }
+      /* Fallthrough */
+    }
     default:
       res = gst_pad_push_event (sinkpad, event);
       break;
@@ -1032,7 +1111,7 @@ gst_video_rate_query (GstBaseTransform * trans, GstPadDirection direction,
         break;
 
       GST_OBJECT_LOCK (videorate);
-      rate = videorate->rate;
+      rate = videorate->pending_rate;
       GST_OBJECT_UNLOCK (videorate);
 
       if (rate == 1.0)
@@ -1245,20 +1324,141 @@ gst_video_rate_check_variable_rate (GstVideoRate * videorate,
   videorate->updating_caps = TRUE;
   gst_base_transform_update_src_caps (GST_BASE_TRANSFORM (videorate), tmpcaps);
 
+  /* also reconfigure sink so that buffer pool can be updated again */
+  gst_base_transform_reconfigure_sink (GST_BASE_TRANSFORM (videorate));
+
 done:
   gst_caps_unref (tmpcaps);
   if (pad)
     gst_object_unref (pad);
 }
 
+static gboolean
+gst_video_rate_switch_mode_if_needed (GstVideoRate * videorate)
+{
+  gboolean switch_mode;
+  GstClockTime avg_period;
+  gboolean skip = FALSE;
+
+  GST_OBJECT_LOCK (videorate);
+  avg_period = videorate->average_period_set;
+  GST_OBJECT_UNLOCK (videorate);
+
+  /* MT-safe switching between modes */
+  if (G_LIKELY (avg_period == videorate->average_period))
+    return skip;
+
+  switch_mode = (avg_period == 0 || videorate->average_period == 0);
+
+  if (!switch_mode)
+    return skip;
+
+
+  videorate->average_period = avg_period;
+  videorate->last_ts = GST_CLOCK_TIME_NONE;
+  if (avg_period) {
+    /* enabling average mode */
+    videorate->average = 0;
+    /* make sure no cached buffers from regular mode are left */
+    gst_video_rate_swap_prev (videorate, NULL, 0);
+  } else {
+    /* enable regular mode */
+    videorate->next_ts = GST_CLOCK_TIME_NONE;
+    skip = TRUE;
+  }
+
+  /* max averaging mode has no latency, normal mode does */
+  gst_element_post_message (GST_ELEMENT (videorate),
+      gst_message_new_latency (GST_OBJECT (videorate)));
+
+  return skip;
+}
+
+static gboolean
+gst_video_rate_do_max_duplicate (GstVideoRate * videorate, GstBuffer * buffer,
+    GstClockTime intime, GstClockTime prevtime, gint * count)
+{
+  if (videorate->max_duplication_time <= 0)
+    return TRUE;
+
+  /* We already know that intime and prevtime are not out of order, based
+   * on the previous condition. Using ABS in case rate < 0, in which case
+   * the order is reversed. */
+  if (ABS (GST_CLOCK_DIFF (intime, prevtime)) > videorate->max_duplication_time) {
+    GST_DEBUG_OBJECT (videorate,
+        "The new buffer (%" GST_TIME_FORMAT
+        ") is further away from previous buffer (%" GST_TIME_FORMAT
+        ") than max-duplication-time (%" GST_TIME_FORMAT ")",
+        GST_TIME_ARGS (intime), GST_TIME_ARGS (prevtime),
+        GST_TIME_ARGS (videorate->max_duplication_time));
+    /* First send out enough buffers to actually reach the time of the
+     * previous buffer */
+    if (videorate->segment.rate < 0.0) {
+      while (videorate->next_ts > prevtime) {
+        gst_video_rate_flush_prev (videorate, *count > 0, GST_CLOCK_TIME_NONE);
+        *count += 1;
+      }
+    } else {
+      while (videorate->next_ts <= prevtime) {
+        gst_video_rate_flush_prev (videorate, *count > 0, GST_CLOCK_TIME_NONE);
+        *count += 1;
+      }
+    }
+
+    if (*count > 1) {
+      videorate->dup += *count - 1;
+      if (!videorate->silent)
+        gst_video_rate_notify_duplicate (videorate);
+    }
+
+    /* The gap between the two buffers is too large. Don't fill it, just
+     * let a discont through */
+    videorate->discont = TRUE;
+
+    if (videorate->segment.rate < 0.0) {
+      videorate->base_ts -= prevtime - intime;
+    } else {
+      videorate->base_ts += intime - prevtime;
+    }
+    videorate->next_ts = intime;
+    /* Swap in new buffer and get rid of old buffer so that starting with
+     * the next input buffer we output from the new position */
+    gst_video_rate_swap_prev (videorate, buffer, intime);
+    return FALSE;
+  }
+
+  return TRUE;
+}
+
+static gboolean
+gst_video_rate_apply_pending_rate (GstVideoRate * videorate)
+{
+  gboolean ret = FALSE;
+
+  GST_OBJECT_LOCK (videorate);
+  if (videorate->pending_rate == videorate->rate)
+    goto done;
+
+  ret = TRUE;
+  videorate->base_ts += gst_util_uint64_scale (videorate->out_frame_count,
+      videorate->to_rate_denominator * GST_SECOND,
+      videorate->to_rate_numerator);
+  videorate->rate = videorate->pending_rate;
+  videorate->out_frame_count = 0;
+
+done:
+  GST_OBJECT_UNLOCK (videorate);
+
+  return ret;
+}
+
 static GstFlowReturn
 gst_video_rate_transform_ip (GstBaseTransform * trans, GstBuffer * buffer)
 {
   GstVideoRate *videorate;
   GstFlowReturn res = GST_BASE_TRANSFORM_FLOW_DROPPED;
   GstClockTime intime, in_ts, in_dur, last_ts;
-  GstClockTime avg_period;
-  gboolean skip = FALSE;
+  gboolean skip;
 
   videorate = GST_VIDEO_RATE (trans);
 
@@ -1269,51 +1469,33 @@ gst_video_rate_transform_ip (GstBaseTransform * trans, GstBuffer * buffer)
 
   if (videorate->to_rate_numerator == 0 && videorate->prevbuf &&
       !videorate->force_variable_rate) {
+    if (!GST_BUFFER_PTS_IS_VALID (buffer) ||
+        !GST_BUFFER_PTS_IS_VALID (videorate->prevbuf)) {
+      GST_ELEMENT_ERROR (videorate, STREAM, FAILED, (NULL),
+          ("videorate requires a non-variable framerate on the output caps or the"
+              " two first consecutive buffers to have valid timestamps to guess the"
+              " framerate."));
+      return GST_FLOW_ERROR;
+    }
     gst_video_rate_check_variable_rate (videorate, buffer);
   }
 
-  GST_OBJECT_LOCK (videorate);
-  avg_period = videorate->average_period_set;
-  GST_OBJECT_UNLOCK (videorate);
-
-  /* MT-safe switching between modes */
-  if (G_UNLIKELY (avg_period != videorate->average_period)) {
-    gboolean switch_mode = (avg_period == 0 || videorate->average_period == 0);
-    videorate->average_period = avg_period;
-    videorate->last_ts = GST_CLOCK_TIME_NONE;
-
-    if (switch_mode) {
-      if (avg_period) {
-        /* enabling average mode */
-        videorate->average = 0;
-        /* make sure no cached buffers from regular mode are left */
-        gst_video_rate_swap_prev (videorate, NULL, 0);
-      } else {
-        /* enable regular mode */
-        videorate->next_ts = GST_CLOCK_TIME_NONE;
-        skip = TRUE;
-      }
-
-      /* max averaging mode has a no latency, normal mode does */
-      gst_element_post_message (GST_ELEMENT (videorate),
-          gst_message_new_latency (GST_OBJECT (videorate)));
-    }
-  }
+  skip = gst_video_rate_switch_mode_if_needed (videorate);
 
   if (videorate->average_period > 0)
     return gst_video_rate_trans_ip_max_avg (videorate, buffer);
 
+  gst_video_rate_apply_pending_rate (videorate);
   in_ts = GST_BUFFER_TIMESTAMP (buffer);
   in_dur = GST_BUFFER_DURATION (buffer);
 
-  if (G_UNLIKELY (in_ts == GST_CLOCK_TIME_NONE)) {
+  if (G_UNLIKELY (!GST_CLOCK_TIME_IS_VALID (in_ts))) {
     /* For reverse playback, we need all input timestamps as we can't
      * guess from the previous buffers timestamp and duration */
-    if (G_UNLIKELY (in_ts == GST_CLOCK_TIME_NONE
-            && videorate->segment.rate < 0.0))
+    if (G_UNLIKELY (videorate->segment.rate < 0.0))
       goto invalid_buffer;
     in_ts = videorate->last_ts;
-    if (G_UNLIKELY (in_ts == GST_CLOCK_TIME_NONE))
+    if (G_UNLIKELY (!GST_CLOCK_TIME_IS_VALID (in_ts)))
       goto invalid_buffer;
   }
 
@@ -1321,7 +1503,7 @@ gst_video_rate_transform_ip (GstBaseTransform * trans, GstBuffer * buffer)
    * next buffer has -1 as a timestamp */
   last_ts = videorate->last_ts;
   videorate->last_ts = in_ts;
-  if (in_dur != GST_CLOCK_TIME_NONE && videorate->segment.rate > 0.0)
+  if (GST_CLOCK_TIME_IS_VALID (in_dur) && videorate->segment.rate > 0.0)
     videorate->last_ts += in_dur;
 
   GST_DEBUG_OBJECT (videorate, "got buffer with timestamp %" GST_TIME_FORMAT,
@@ -1351,18 +1533,36 @@ gst_video_rate_transform_ip (GstBaseTransform * trans, GstBuffer * buffer)
        * timestamp in the segment */
       if (videorate->skip_to_first || skip) {
         videorate->next_ts = intime;
-        if (videorate->segment.rate < 0.0)
+        if (videorate->segment.rate < 0.0) {
           videorate->base_ts = videorate->segment.stop - in_ts;
-        else
+        } else {
           videorate->base_ts = in_ts - videorate->segment.start;
+        }
         videorate->out_frame_count = 0;
       } else {
-        if (videorate->segment.rate < 0.0)
-          videorate->next_ts =
-              videorate->segment.stop + videorate->segment.base;
-        else
+        if (videorate->segment.rate < 0.0) {
+          if (videorate->to_rate_numerator) {
+            GstClockTime frame_duration = gst_util_uint64_scale (1,
+                videorate->to_rate_denominator * GST_SECOND,
+                videorate->to_rate_numerator);
+
+            videorate->next_ts =
+                videorate->segment.stop + videorate->segment.base;
+
+            if (videorate->next_ts > frame_duration)
+              videorate->next_ts =
+                  MAX (videorate->segment.start,
+                  videorate->next_ts - frame_duration);
+            else
+              videorate->next_ts = videorate->segment.start;
+          } else {
+            /* What else can we do? */
+            videorate->next_ts = intime;
+          }
+        } else {
           videorate->next_ts =
               videorate->segment.start + videorate->segment.base;
+        }
       }
     }
 
@@ -1375,8 +1575,11 @@ gst_video_rate_transform_ip (GstBaseTransform * trans, GstBuffer * buffer)
           (videorate->segment.rate < 0.0 && intime <= videorate->next_ts)) {
         GstFlowReturn r;
 
-        /* on error the _flush function posted a warning already */
-        if ((r = gst_video_rate_flush_prev (videorate, FALSE,
+        /* The buffer received from basetransform is guaranteed to be writable.
+         * It just needs to be reffed so the buffer won't be consumed once pushed and
+         * GstBaseTransform can get its reference back. */
+        if ((r = gst_video_rate_push_buffer (videorate,
+                    gst_buffer_ref (buffer), FALSE,
                     GST_CLOCK_TIME_NONE)) != GST_FLOW_OK) {
           res = r;
           goto done;
@@ -1388,7 +1591,7 @@ gst_video_rate_transform_ip (GstBaseTransform * trans, GstBuffer * buffer)
   } else {
     GstClockTime prevtime;
     gint count = 0;
-    gint64 diff1, diff2;
+    gint64 diff1 = 0, diff2 = 0;
 
     prevtime = videorate->prev_ts;
 
@@ -1413,39 +1616,89 @@ gst_video_rate_transform_ip (GstBaseTransform * trans, GstBuffer * buffer)
       goto done;
     }
 
+    if (!gst_video_rate_do_max_duplicate (videorate, buffer, intime, prevtime,
+            &count))
+      goto done;
+
     /* got 2 buffers, see which one is the best */
     do {
-      GstClockTime next_ts = videorate->next_ts * videorate->rate;
+      GstClockTime next_ts;
 
-      /* take absolute values, beware: abs and ABS don't work for gint64 */
-      if (prevtime > next_ts)
-        diff1 = prevtime - next_ts;
-      else
-        diff1 = next_ts - prevtime;
-
-      if (intime > next_ts)
-        diff2 = intime - next_ts;
-      else
-        diff2 = next_ts - intime;
-
-      GST_LOG_OBJECT (videorate,
-          "diff with prev %" GST_TIME_FORMAT " diff with new %"
-          GST_TIME_FORMAT " outgoing ts %" GST_TIME_FORMAT,
-          GST_TIME_ARGS (diff1), GST_TIME_ARGS (diff2),
-          GST_TIME_ARGS (next_ts));
+      if (gst_video_rate_apply_pending_rate (videorate))
+        goto done;
 
       if (videorate->segment.rate < 0.0) {
         /* Make sure that we have a duration for this buffer. The previous
          * buffer already has a duration given by either exactly this code,
          * or the code above for the very first buffer */
         g_assert (GST_BUFFER_DURATION_IS_VALID (videorate->prevbuf));
-        if (!GST_BUFFER_DURATION_IS_VALID (buffer) && prevtime > intime)
-          GST_BUFFER_DURATION (buffer) = prevtime - intime;
+        if (!GST_BUFFER_DURATION_IS_VALID (buffer))
+          GST_BUFFER_DURATION (buffer) =
+              prevtime > intime ? prevtime - intime : 0;
       } else {
         /* Make sure that we have a duration for previous buffer */
-        if (!GST_BUFFER_DURATION_IS_VALID (videorate->prevbuf)
-            && intime > prevtime)
-          GST_BUFFER_DURATION (videorate->prevbuf) = intime - prevtime;
+        if (!GST_BUFFER_DURATION_IS_VALID (videorate->prevbuf))
+          GST_BUFFER_DURATION (videorate->prevbuf) =
+              intime > prevtime ? intime - prevtime : 0;
+      }
+
+#ifndef ABSDIFF
+#define ABSDIFF(a, b) (((a) > (b)) ? (a) - (b) : (b) - (a))
+#endif
+
+      /* take absolute diffs */
+      if (videorate->segment.rate < 0.0) {
+        GstClockTime next_end_ts;
+        GstClockTime prev_endtime;
+        GstClockTime in_endtime, base_ts_in_segment;
+
+        next_ts = videorate->next_ts;
+
+        if (!GST_CLOCK_TIME_IS_VALID (next_ts)) {
+          GST_DEBUG_OBJECT (videorate, "Already reached segment start,"
+              "ignoring buffer");
+          break;
+        }
+
+        prev_endtime = prevtime + GST_BUFFER_DURATION (videorate->prevbuf);
+        in_endtime = intime + GST_BUFFER_DURATION (buffer);
+
+        if (videorate->to_rate_numerator) {
+          GstClockTime frame_duration = gst_util_uint64_scale (1,
+              videorate->to_rate_denominator * GST_SECOND,
+              videorate->to_rate_numerator);
+          next_end_ts = next_ts + frame_duration;
+        } else {
+          next_end_ts = next_ts + GST_BUFFER_DURATION (videorate->prevbuf);
+        }
+
+        base_ts_in_segment = videorate->segment.stop - videorate->base_ts;
+        next_ts = base_ts_in_segment - (
+            (base_ts_in_segment - next_ts) * videorate->rate);
+        next_end_ts = base_ts_in_segment - (MAX (0,
+                (base_ts_in_segment - next_end_ts)) * videorate->rate);
+
+        diff1 = ABSDIFF (prev_endtime, next_end_ts);
+        diff2 = ABSDIFF (in_endtime, next_end_ts);
+
+        GST_LOG_OBJECT (videorate,
+            "diff with prev %" GST_TIME_FORMAT " diff with new %"
+            GST_TIME_FORMAT " outgoing ts %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (diff1), GST_TIME_ARGS (diff2),
+            GST_TIME_ARGS (next_end_ts));
+      } else {
+        next_ts =
+            videorate->base_ts + ((videorate->next_ts -
+                videorate->base_ts) * videorate->rate);
+
+        diff1 = ABSDIFF (prevtime, next_ts);
+        diff2 = ABSDIFF (intime, next_ts);
+
+        GST_LOG_OBJECT (videorate,
+            "diff with prev %" GST_TIME_FORMAT " diff with new %"
+            GST_TIME_FORMAT " outgoing ts %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (diff1), GST_TIME_ARGS (diff2),
+            GST_TIME_ARGS (next_ts));
       }
 
       /* output first one when its the best */
@@ -1466,7 +1719,7 @@ gst_video_rate_transform_ip (GstBaseTransform * trans, GstBuffer * buffer)
     }
     while (diff1 < diff2);
 
-    /* if we outputed the first buffer more then once, we have dups */
+    /* if we outputted the first buffer more then once, we have dups */
     if (count > 1) {
       videorate->dup += count - 1;
       if (!videorate->silent)
@@ -1570,11 +1823,14 @@ gst_video_rate_set_property (GObject * object,
       g_atomic_int_set (&videorate->max_rate, g_value_get_int (value));
       goto reconfigure;
     case PROP_RATE:
-      videorate->rate = g_value_get_double (value);
+      videorate->pending_rate = g_value_get_double (value);
       GST_OBJECT_UNLOCK (videorate);
 
       gst_videorate_update_duration (videorate);
       return;
+    case PROP_MAX_DUPLICATION_TIME:
+      videorate->max_duplication_time = g_value_get_uint64 (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1632,7 +1888,10 @@ gst_video_rate_get_property (GObject * object,
       g_value_set_int (value, g_atomic_int_get (&videorate->max_rate));
       break;
     case PROP_RATE:
-      g_value_set_double (value, videorate->rate);
+      g_value_set_double (value, videorate->pending_rate);
+      break;
+    case PROP_MAX_DUPLICATION_TIME:
+      g_value_set_uint64 (value, videorate->max_duplication_time);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
